/* part of the shansyn spherical harmonics package, see COPYRIGHT for license */
/* $Id: abconvert.c,v 1.31 2004/06/22 00:31:24 becker Exp becker $ */
/* 
   program does tapering and format conversion for
   spherical harmonic coefficients 

   handles physical convention 
   and geodetic normalization

*/
#include "abconvert.h"


int main(int argc, char *argv[] )
{
  int lmax,lmax1,l,m,tapering,out_format,lmsize,in_format,
    llmax,layers=0,i,highpass,verbose=VERBOSE,velin=FALSE,
    os1;
  COMP_PRECISION *a=NULL,*b=NULL,tmp,amplitudefactor,
    *filter=NULL,rf1,rf2,dp,atmp,btmp,*amt=NULL,*amp=NULL,
    *bmp=NULL,*bmt=NULL,*c=NULL,*d=NULL,*cmt=NULL,*cmp=NULL,
    *dmp=NULL,*dmt=NULL,lc,weight1,weight2,
    fac1,fac2,fac3;
  FILE *in=NULL;
  if(argc > 1 && strcmp(argv[1],"-h")==0 )
    argc=-9;

  /* default values */
  in_format=IN_FORMAT_DEFAULT;
  out_format=OUT_FORMAT_DEFAULT;
  tapering=TAPERING_DEFAULT;
  amplitudefactor=AMPLITUDE_DEFAULT;
  lc=LC_DEFAULT;

  highpass=0;
  llmax=99999;
  
  switch(argc){
  case 1:{
    break;
  }
  case 2:{
    sscanf(argv[1],"%i",&out_format);
    break;
  }
  case 3:{
    sscanf(argv[1],"%i",&out_format);
    sscanf(argv[2],"%i",&tapering);
    break;
  }
  case 4:{
    sscanf(argv[1],"%i",&out_format);
    sscanf(argv[2],"%i",&tapering);
    sscanf(argv[3],DATA_FSCAN_FORMAT,&amplitudefactor);
    break;
  }
  case 5:{
    sscanf(argv[1],"%i",&out_format);
    sscanf(argv[2],"%i",&tapering);
    sscanf(argv[3],DATA_FSCAN_FORMAT,&amplitudefactor);
    sscanf(argv[4],"%i",&in_format);
    break;
  }
  case 6:{
    sscanf(argv[1],"%i",&out_format);
    sscanf(argv[2],"%i",&tapering);
    sscanf(argv[3],DATA_FSCAN_FORMAT,&amplitudefactor);
    sscanf(argv[4],"%i",&in_format);
    sscanf(argv[5],"%i",&llmax);
    if(llmax==0){
      llmax=99999;
    }else if(llmax < 0){
      highpass= -llmax;
      llmax=99999;
    }
    break;
  }
  case 7:{
    sscanf(argv[1],"%i",&out_format);
    sscanf(argv[2],"%i",&tapering);
    sscanf(argv[3],DATA_FSCAN_FORMAT,&amplitudefactor);
    sscanf(argv[4],"%i",&in_format);
    sscanf(argv[5],"%i",&llmax);
    if(llmax==0){
      llmax=99999;
    }else if(llmax < 0){
      highpass= -llmax;
      llmax=99999;
    }
    sscanf(argv[6],DATA_FSCAN_FORMAT,&lc);
    if(lc >= 1.0){
      fprintf(stderr,"%s: WARNING: lc >= 1: switching tapering off. is that what you wanted?\n",
	      argv[0]);
      lc=0.0;
      tapering=NO_TAPER;
    }
    break;
  }
  default:{
    phelp(argv[0]);
    exit(-1);
  }}
  /*
    open filter file, if required
  */
  if(tapering == FROM_FILE_TAPER){
    lc=0.0;
    fprintf(stderr,"%s: resetting l_c to zero for filtering\n",argv[0]);
    in=fopen(FILTER_FILE,"r");
    if(!in){
      fprintf(stderr,"%s: could not read %s for filter weights\n",
	      argv[0],FILTER_FILE);
      fprintf(stderr,"%s: taper option %i was set\n",
	      argv[0],tapering);
      exit(-1);
    }
  }
  /*

    do some sanity checks 

  */
  if((in_format == ABAB_INPUT)||(in_format == ABAB_GEOD_INPUT) || 
     (in_format == AABBR_INPUT)||(in_format == INTERPOLATE_ABAB)){// vector field input
    if((out_format ==  ABPHYS_OUT)||(out_format ==  ABGEOD_OUT) ||
       (out_format == LMAB_OUT)||(out_format == LMAB_GEODETIC_OUT)){
      fprintf(stderr,"%s: mixing vector field input with scalar output\n",
	      argv[0]);
      phelp(argv[0]);exit(-1);
    }
  }else{// scalar field input
    if((out_format == VECABAB_OUT)||(out_format== VECAABBR_OUT)){
      fprintf(stderr,"%s: mixing scalar field input with vector output\n",
	      argv[0]);
      phelp(argv[0]);exit(-1);
    }
  }
  /*
    
    DATA INPUT

    
    read in set(s) of coefficients and convert to internal 
    convention 

    start of input loop 
    
  */
  while(fscanf(stdin,"%i",&lmax) == 1){
    lmax1 = lmax+1;
    switch(in_format){// start input format case structure
    case INTERPOLATE:
    case AB_INPUT:
      if(verbose)
	fprintf(stderr,"%s: reading A B format in physical convention\n",argv[0]);
      lmsize= (int)((((float)lmax)+1.0)*(((float)lmax)+2)/2.0);
      if((a=(COMP_PRECISION *)
	  calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	 (b=(COMP_PRECISION *)
	  calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",
		argv[0],lmax,lmsize); 
	phelp(argv[0]);exit(-1);
      }
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  if((fscanf(stdin,TWO_DATA_FSCAN_FORMAT,
		     (a+os1),(b+os1)))!=2){
	    fprintf(stderr,"%s: read error, l=%i m=%i\n\n",
		    argv[0],l,m);exit(-1);}
	}
      break;
    case LAB_GEOD_INPUT:// layer lm A B (different from lm A B geodetic!!
      if(verbose)
	fprintf(stderr,"%s: reading l m A B LAYER FORMAT in GEODETIC convention\n",argv[0]);
      
      lmsize= (int)((((float)lmax)+1.0)*(((float)lmax)+2)/2.0);
      if((a=(COMP_PRECISION *)
	  calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	 (b=(COMP_PRECISION *)
	  calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",
		argv[0],lmax,lmsize); 
	phelp(argv[0]);exit(-1);}
      if(layers!=1)
	if(verbose)
	  fprintf(stderr,"%s: data file has %i layers, reading only layer 1\n",argv[0],layers);
      for(m=0;m<layers;m++) 
	fscanf(stdin,"%*g");
      fscanf(stdin,"%*i %*g");
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1=POSLM(l, m);
	  if((fscanf(stdin,TWO_TWO_DATA_FSCAN_FORMAT,
		     (a+os1),(b+os1)))!=2){
	    fprintf(stderr,"%s: read error, l=%i m=%i\n\n",argv[0],l,m);exit(-1);}
	  *(a+os1) *= (GEODETIC_FACTOR(l,m));
	  *(b+os1) *= (GEODETIC_FACTOR(l,m));
	}
      break;
    case AB_GEOD_INPUT:{
      if(verbose)
	fprintf(stderr,"%s: reading A B format in geodetic convention \n",argv[0]);
      lmsize= (int)((((float)lmax)+1.0)*(((float)lmax)+2)/2.0);
      if((a=(COMP_PRECISION *)
	  calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	 (b=(COMP_PRECISION *)
	  calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	if(verbose)fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",argv[0],lmax,lmsize); 
	phelp(argv[0]);exit(-1);}
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  if((fscanf(stdin,TWO_DATA_FSCAN_FORMAT,(a+os1),(b+os1)))!=2){
	    fprintf(stderr,"%s: read error, l=%i m=%i\n\n",argv[0],l,m);exit(-1);}
	  *(a+os1) *= (GEODETIC_FACTOR(l,m));
	  *(b+os1) *= (GEODETIC_FACTOR(l,m));
	}
      break;
    }
    case AB_RICK_INPUT:{
      if(verbose)
	fprintf(stderr,"%s: reading A B format in Rick's convention \n",argv[0]);
      lmsize= (int)((((float)lmax)+1.0)*(((float)lmax)+2)/2.0);
      if((a=(COMP_PRECISION *)
	  calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	 (b=(COMP_PRECISION *)
	  calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	if(verbose)fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",argv[0],lmax,lmsize); 
	phelp(argv[0]);exit(-1);}
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  if((fscanf(stdin,TWO_DATA_FSCAN_FORMAT,(a+os1),(b+os1)))!=2){
	    fprintf(stderr,"%s: read error, l=%i m=%i\n\n",argv[0],l,m);exit(-1);}
	  *(a+os1) *= (RICK_SCALAR_FACTOR(l,m));
	  *(b+os1) *= (RICK_SCALAR_FACTOR(l,m));
	}
      break;
    }
    case AB_MASTERS_INPUT:{
      if(verbose)
	fprintf(stderr,"%s: reading A B format in G. Master's (Edmonds, 1960) convention \n",argv[0]);
      lmsize= (int)((((float)lmax)+1.0)*(((float)lmax)+2)/2.0);
      if((a=(COMP_PRECISION *)
	  calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	 (b=(COMP_PRECISION *)
	  calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	if(verbose)fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",argv[0],lmax,lmsize); 
	phelp(argv[0]);exit(-1);}
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  if((fscanf(stdin,TWO_DATA_FSCAN_FORMAT,(a+os1),(b+os1)))!=2){
	    fprintf(stderr,"%s: read error, l=%i m=%i\n\n",argv[0],l,m);exit(-1);}
	  *(a+os1) *=  SQRT_TWO; 
	  *(b+os1) *= -SQRT_TWO;
	}
      break;
    }
    case INTERPOLATE_ABAB:// velocity ABAB format input
    case ABAB_INPUT:{
      if(verbose)
	fprintf(stderr,
		"%s: reading poltor vector AB AB format in physical convention\n",
		argv[0]);
      lmsize= (int)((((float)lmax)+1.0)*(((float)lmax)+2)/2.0);
      if((amt=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	 (bmt=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	if(verbose)fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",argv[0],lmax,lmsize); 
	phelp(argv[0]);exit(-1);}
      if((amp=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	 (bmp=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",argv[0],lmax,lmsize); 
	phelp(argv[0]);exit(-1);}
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){ 
	  os1 = POSLM(l, m);
	  if((fscanf(stdin,FOUR_DATA_FSCAN_FORMAT,
		     (amp+os1),(bmp+os1),(amt+os1),(bmt+os1)))!=4){
	    fprintf(stderr,"%s: read error, A l=%i m=%i\n\n",argv[0],l,m);
	    exit(-1);
	  }
	}
      velin=TRUE;
      break;
    }
    case ABAB_GEOD_INPUT:{
      if(verbose)
	fprintf(stderr,"%s: reading poltor vector AB AB format in geodetic convention\n",argv[0]);
      
      lmsize= (int)((((float)lmax)+1.0)*(((float)lmax)+2)/2.0);
      if((amt=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	 (bmt=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",argv[0],lmax,lmsize); 
	phelp(argv[0]);exit(-1);}
      if((amp=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	 (bmp=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",argv[0],lmax,lmsize); 
	phelp(argv[0]);exit(-1);}
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){	  
	  os1 = POSLM(l, m);
	  if((fscanf(stdin,FOUR_DATA_FSCAN_FORMAT,
		     (amp+os1),(bmp+os1),(amt+os1),
		     (bmt+os1)))!=4){
	    fprintf(stderr,"%s: read error, A l=%i m=%i\n\n",argv[0],l,m);exit(-1);}
	  *(amp+os1) *= GEODETIC_FACTOR(l, m);
	  *(bmp+os1) *= GEODETIC_FACTOR(l, m);
	  *(amt+os1) *= GEODETIC_FACTOR(l, m);
	  *(bmt+os1) *= GEODETIC_FACTOR(l, m);
	}
      velin=TRUE;
      break;
    }
    case AABBR_INPUT:{
      if(verbose)
	fprintf(stderr,"%s: reading poltor vector AA\\nBB format in Rick's convention\n",argv[0]);
      lmsize= (int)((((float)lmax)+1.0)*(((float)lmax)+2)/2.0);
      if((amt=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	 (bmt=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	if(verbose)fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",argv[0],lmax,lmsize); 
	phelp(argv[0]);exit(-1);}
      if((amp=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	 (bmp=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	if(verbose)fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",argv[0],lmax,lmsize); 
	phelp(argv[0]);exit(-1);}
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  if(fscanf(stdin,TWO_DATA_FSCAN_FORMAT,(amp+os1),(amt+os1))!=2){
	    if(verbose)fprintf(stderr,"%s: read error, A l=%i m=%i\n\n",argv[0],l,m);exit(-1);}
	  if(fscanf(stdin,TWO_DATA_FSCAN_FORMAT,(bmp+os1),(bmt+os1))!=2){
	    if(verbose)fprintf(stderr,"%s: read error, B l=%i m=%i\n\n",argv[0],l,m);exit(-1);}
	  *(amp+os1) *= RICK_FACTOR(l, m);
	  *(bmp+os1) *= RICK_FACTOR(l, m);
	  *(amt+os1) *= RICK_FACTOR(l, m);
	  *(bmt+os1) *= RICK_FACTOR(l, m);
	}
      velin=TRUE;
      break;
    }
    case LMAB_GEOD_INPUT:{
      if(verbose)
	fprintf(stderr,"%s: reading l m A B format in geodetic convention \n",argv[0]);
      
      lmsize= (int)((((float)lmax)+1.0)*(((float)lmax)+2)/2.0);
      if((a=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	 (b=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	if(verbose)fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",argv[0],lmax,lmsize); 
	phelp(argv[0]);exit(-1);}
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  if((fscanf(stdin,TWO_TWO_DATA_FSCAN_FORMAT,
		     (a+os1),(b+os1)))!=2){
	    if(verbose)fprintf(stderr,"%s: read error, l=%i m=%i\n\n",argv[0],l,m);exit(-1);}
	  *(a+os1) *= GEODETIC_FACTOR(l, m);
	  *(b+os1) *= GEODETIC_FACTOR(l, m);
	}
      break;
    }
    case LMAB_FNORM_INPUT:{
      if(verbose)
	fprintf(stderr,"%s: reading l m A B in fully normalized (Edmonds, 1960) format\n",argv[0]);
      
      lmsize= (int)((((float)lmax)+1.0)*(((float)lmax)+2)/2.0);
      if((a=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	 (b=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	if(verbose)fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",argv[0],lmax,lmsize); 
	phelp(argv[0]);exit(-1);}
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  if((fscanf(stdin,TWO_TWO_DATA_FSCAN_FORMAT,
		     (a+os1),(b+os1)))!=2){
	    if(verbose)fprintf(stderr,"%s: read error, l=%i m=%i\n\n",argv[0],l,m);exit(-1);}
	}
      // go from "fully normalized" (Edmonds or old Harvard)  to Rick's convention
      // (this from advect code....)
      fac1 = 1.0/sqrt(4.0*PI);
      fac2 = fac1/sqrt(2.0);
      for(l=0;l<=lmax;l++){
	a[POSLM(l,0)] *= fac1;
	if(l>0){
	  for(m=1;m<=l;m++){
	    os1 = POSLM(l,m);
	    fac3 = fac2 * pow(-1.0,(COMP_PRECISION)(m));
	    a[os1] *= fac3;
	    b[os1] *= fac3;
	  }
	}
      }
      // go from Rick's convention to our physical norm
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  a[os1] *= RICK_SCALAR_FACTOR(l,m);
	  b[os1] *= RICK_SCALAR_FACTOR(l,m);
	}
      break;
    }
    default:{
      phelp(argv[0]);exit(-1);
      break;
    }
    }// end of input format case structure
    //
    //
    //
    // output/input formats might require second file
    //
    if(out_format == CORRL_OUT || out_format == COORT_OUT ||
       in_format == INTERPOLATE || in_format == INTERPOLATE_ABAB){
      if(verbose){
	if(in_format == INTERPOLATE || in_format == INTERPOLATE_ABAB)
	  fprintf(stderr,"%s: second %s file for interpolation, lmax=%i\n",
		  argv[0],(in_format==INTERPOLATE)?"AB":"ABAB",lmax);
	else
	  fprintf(stderr,"%s: second file for correlation coefficient, lmax=%i\n",argv[0],lmax);
      }
      fscanf(stdin,"%i",&i);// second lmax
      if(i != lmax){
	if(verbose)fprintf(stderr,"%s: second lmax=%i not equal to first file lmax=%i\n\n",argv[0],i,lmax);
	exit(-1);
      }
      if(in_format != INTERPOLATE_ABAB){// read in scalar expansion
	if((c=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	   (d=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	  if(verbose)fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",argv[0],lmax,lmsize); 
	  phelp(argv[0]);exit(-1);}
	for(l=0;l<=lmax;l++)
	  for(m=0;m<=l;m++){
	    os1 = POSLM(l, m);
	    if((fscanf(stdin,TWO_DATA_FSCAN_FORMAT,(c+os1),(d+os1)))!=2){
	      if(verbose)fprintf(stderr,"%s: second file read error, l=%i m=%i\n\n",argv[0],l,m);exit(-1);}
	  }
      }else{// read in velocity expansion
	if((cmt=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	   (dmt=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	  if(verbose)fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",argv[0],lmax,lmsize); 
	  phelp(argv[0]);exit(-1);}
	if((cmp=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL ||
	   (dmp=(COMP_PRECISION *)calloc(lmsize,sizeof(COMP_PRECISION)))==NULL){
	  fprintf(stderr,"%s: memerror, lmax=%i lmsize=%i\n",argv[0],lmax,lmsize); 
	  phelp(argv[0]);exit(-1);}
	for(l=0;l<=lmax;l++)
	  for(m=0;m<=l;m++){
	    os1 = POSLM(l, m);
	    if((fscanf(stdin,FOUR_DATA_FSCAN_FORMAT,
		       (cmp+os1),(dmp+os1),
		       (cmt+os1),(dmt+os1)))!=4){
	      fprintf(stderr,"%s: read error, second ABAB file l=%i m=%i\n\n",argv[0],l,m);
	      exit(-1);
	    }
	  }
      }
    }// end second expansion file input loop
    //
    // 
    //
    if(tapering == FROM_FILE_TAPER){
      //
      // read in filter weights
      //
      filter=(COMP_PRECISION *)malloc(sizeof(COMP_PRECISION)*
				      (lmax1));
      if(!filter)
	MEMERROR;
      for(i=0;i<=lmax;i++)
	if(fscanf(in,DATA_FSCAN_FORMAT,(filter+i))!=1){
	  fprintf(stderr,"%s: read error, weight l=%i of %s\n",
		  argv[0],i,FILTER_FILE);
	  exit(-1);
	}
      fclose(in);
      if(verbose)
	fprintf(stderr,"%s: succesfully read %i filter weights\n",
		argv[0],lmax1);
    } /* end from file taper */
    /* 
       

       APPLY TAPER
     

    */
    switch(tapering){
    case NO_TAPER:{/* no taper */
      break;
    }
    case COSSQR_TAPER:{
      if(verbose)
	fprintf(stderr,"%s: applying cos^2 taper  from l' > %g\n",argv[0],lc);
      break;
    }
    case COSP4_TAPER:{
      if(verbose)
	fprintf(stderr,"%s: applying cos^4 taper  from l' > %g\n",argv[0],lc);
      break;
    }
    case ONEML_TAPER:{
      if(verbose)
	fprintf(stderr,"%s: applying linear taper from l' > %g\n",argv[0],lc);
      break;
    }
    case ONEMLSQR_TAPER:{
      if(verbose)
	fprintf(stderr,"%s: applying square taper from l' > %g\n",argv[0],lc);
      break;
    }
    case SINOVRL_TAPER:{
      if(verbose)
	fprintf(stderr,"%s: applying Lanczos taper  from l' > %g\n",argv[0],lc);
      break;
    }
    case SINOVRLM_TAPER:{
      if(verbose)
	fprintf(stderr,"%s: applying Forte taper for l and m\n",argv[0]);
      break;
    }
    case NNR_TAPER:{
      if(verbose)
	fprintf(stderr,"%s: removing net rotation part of pol/tor AB set\n",argv[0]);
      break;
    }
    case ZERO_TAPER:{
      if(verbose)
	fprintf(stderr,"%s: setting all coefficients >= l' (%g = %i) to zero\n",argv[0],
		lc,(int)(lc*(float)lmax1));
      break;
    }
    case L0_TAPER:{
      if(verbose)
	fprintf(stderr,"%s: removing l=0 term (mean=0)\n",argv[0]);
      break;
    }
    case FROM_FILE_TAPER:{
      if(verbose)
	fprintf(stderr,"%s: applying filter\n",argv[0]);
      break;
    }
    case PHI_ROTATE:{
      if(verbose)
	fprintf(stderr,"%s: rotating field by %g degrees eastward\n",
		argv[0],amplitudefactor);
      break;
    }
    default:{
      if(verbose)
	fprintf(stderr,"%s: tapering mode %i is not defined\n",argv[0],tapering);
      phelp(argv[0]);
      exit(-1);
      break;
    }}				/* end tapering switch */
    if((in_format == INTERPOLATE) || (in_format == INTERPOLATE_ABAB)){
      /*
	
	input format needs interpolation between files
	in this case, amp factor is the weight

      */
      weight1 = amplitudefactor;
      weight2 = 1.0 - amplitudefactor;
      if(verbose)
	fprintf(stderr,"%s: first/second set are weighted by %g/%g\n",
		argv[0],weight1,weight2);
      if(weight1 > 1 || weight1 < 0){
	fprintf(stderr,"%s: first weight not between 0 and 1\n",
		argv[0]);
      }
      if(in_format == INTERPOLATE)// AB FORMAT
	for(l=0;l<=lmax;l++)
	  for(m=0;m<=l;m++){
	    os1 = POSLM(l, m);
	    *(a+os1) *= weight1;
	    *(a+os1) += *(c+os1) * weight2;
	    *(b+os1) *= weight1;
	    *(b+os1) += *(d+os1) * weight2;
	  }
      else// ABAB format 
	for(l=0;l<=lmax;l++)
	  for(m=0;m<=l;m++){
	    os1 = POSLM(l, m);
	    *(amp+os1) *= weight1;
	    *(amp+os1) += *(cmp+os1) * weight2;
	    *(bmp+os1) *= weight1;
	    *(bmp+os1) += *(dmp+os1) * weight2;
	    *(amt+os1) *= weight1;
	    *(amt+os1) += *(cmt+os1) * weight2;
	    *(bmt+os1) *= weight1;
	    *(bmt+os1) += *(dmt+os1) * weight2;
	  }
      amplitudefactor = 1.0;
    }else if(tapering==PHI_ROTATE){
      /*
	
	if tapering is rotation, then amplitude factor is 
	the phi shift in degrees
	apply rotation here

      */
      dp=amplitudefactor*PIOVERONEEIGHTY;
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  tmp = (COMP_PRECISION)m*dp;
	  rf1=cos(tmp);
	  rf2=sin(tmp);
	  atmp=rf1* *(a+os1) - rf2 * *(b+os1);
	  btmp=rf2* *(a+os1) + rf1 * *(b+os1);
	  *(a+os1)=atmp;
	  *(b+os1)=btmp;
	}
      amplitudefactor=1.0;
    }else{
      if(verbose && (amplitudefactor != 1.0))
	fprintf(stderr,"%s: amplifying A and B by %g\n",argv[0],amplitudefactor);
      if(amplitudefactor == 0.0)
	fprintf(stderr,"%s: WARNING: scaling by zero!\n",argv[0]);
    }
    /*
      
      since we have interpolate, we do not need to care about the 
      second expansion anymore
      
      now apply taper
      
    */
    if(highpass)
      if(verbose)
	fprintf(stderr,"%s: all coefficients below l=%i are set to zero\n",argv[0],highpass);
    if(!velin){// apply taper for AB
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  tmp = amplitudefactor * 
	    taperf(l,lmax,lc,tapering,m,filter);
	  if(l < highpass)
	    tmp=0.0;
	  *(a+os1) *= tmp; *(b+os1) *= tmp;
	}
    }else{
      if(tapering == NNR_TAPER){/* 
				   NNR and other tapers are NOT
				   EXCLUSIVE since highpass might 
				   still apply! 
				*/
	*(amt+POSLM(1, 0))=0.0;
	*(amt+POSLM(1, 1))=0.0;
	*(bmt+POSLM(1, 1))=0.0;
      }
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  tmp=amplitudefactor*
	    taperf(l,lmax,lc,tapering,m,filter);
	  if(l < highpass)tmp=0.0;
	  *(amp+os1) *= tmp;
	  *(bmp+os1) *= tmp;
	  *(amt+os1) *= tmp;
	  *(bmt+os1) *= tmp;
	}
    }
    /* 
       
       output 


       change l_max

    */
    if(llmax != 99999){
      if(llmax > lmax){
	if(velin){
	  fprintf(stderr,"%s: expanding coefficients to new l_max only implemented for single set\n",
		  argv[0]);
	  exit(-1);
	}
	if(verbose)
	  fprintf(stderr,"%s: new lmax value (%i) with original lmax (%i), fill up with zeroes\n",
		  argv[0],llmax,lmax);
	lmsize=(int)((((float)llmax)+1.0)*(((float)llmax)+2)/2.0);
	if((a=(COMP_PRECISION *)realloc(a,lmsize*sizeof(COMP_PRECISION)))==NULL ||
	   (b=(COMP_PRECISION *)realloc(b,lmsize*sizeof(COMP_PRECISION)))==NULL){
	  fprintf(stderr,
		  "%s: memerror while resizing lmax=%i lmsize=%i\n",
		  argv[0],llmax,lmsize); 
	  exit(-1);
	}
	for(l=lmax+1;l<=llmax;l++)
	  for(m=0;m<=l;m++){
	    os1 = POSLM(l,m);
	    a[os1]=b[os1]=0.0;
	  }
	lmax=llmax;
      }else if(llmax<0){
	fprintf(stderr,"%s: new lmax is negative (%i), that's nonsense\n",argv[0],lmax);
	phelp(argv[0]);exit(-1);
      }else if(lmax!=llmax){
	if(verbose)fprintf(stderr,"%s: limiting output from %i to lmax=%i\n",argv[0],lmax,llmax);    
	lmax=llmax;
      }
    } /* end lmax != 99999 */
    switch(out_format){// begin output format case structure
    case ABPHYS_OUT:{
      if(verbose)
	fprintf(stderr,"%s: physical convention AB format output, lmax=%i\n",argv[0],lmax);
      write_coefficients(a,b,lmax,1.0,TRUE);
      break;
    }
    case ABGEOD_OUT:{
      if(verbose)fprintf(stderr,"%s: geodetic convention A'B' format output, lmax=%i\n",argv[0],lmax);
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  fprintf(stdout,"%21.14e\n",
		  *(a+os1)/GEODETIC_FACTOR(l,m));
	}
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  fprintf(stdout,"%21.14e\n",
		  *(b+os1)/GEODETIC_FACTOR(l,m));
	}
      break;
    }
    case POWER_OUT:{ 
      if(!velin){
	if(verbose)
	  fprintf(stderr,"%s: output: l power_per_degree_per_unit_area, lmax=%i\n",
		  argv[0],lmax);
	for(l=0;l<=lmax;l++)
	  fprintf(stdout,"%i %21.14e\n",l,degree_power(a,b,l));
      }else{
	if(verbose)
	  fprintf(stderr,"%s: output: l pwr/degree/unit_area|_pol  pwr/degree/unit_area|_tor, lmax=%i\n",
		  argv[0],lmax);
	for(l=0;l<=lmax;l++)
	  fprintf(stdout,"%i %21.14e %21.14e\n",
		  l,degree_power(amp,bmp,l),degree_power(amt,bmt,l));
      }
      break;
    }
    case TRMS_OUT:{ 
      if(!velin){
	if(verbose)
	  fprintf(stderr,"%s: output: RMS of expansion, lmax=%i\n",
		  argv[0],lmax);
	fprintf(stdout,"%21.14e\n",calc_rms(a,b,lmax));
      }else{
	if(verbose)
	  fprintf(stderr,"%s: output: RMS/unit_area|_pol  RMS/unit_area|_tor, lmax=%i\n",
		  argv[0],lmax);
	fprintf(stdout,"%21.14e %21.14e\n",
		calc_rms(amp,bmp,lmax),calc_rms(amt,bmt,lmax));
      }
      break;
    }
    case TPOWER_OUT:{ 
      if(!velin){
	if(verbose)
	  fprintf(stderr,"%s: output: magnitude (sqrt(total power)) of expansion, lmax=%i\n",
		  argv[0],lmax);
	fprintf(stdout,"%21.14e\n",calc_total_power(a,b,lmax));
      }else{
	if(verbose)
	  fprintf(stderr,"%s: output: RMS/unit_area|_pol  RMS/unit_area|_tor, lmax=%i\n",
		  argv[0],lmax);
	fprintf(stdout,"%21.14e %21.14e\n",
		calc_total_power(amp,bmp,lmax),calc_total_power(amt,bmt,lmax));
      
      }
      break;
    }
    case TAPER_OUT:{
      if(verbose)
	fprintf(stderr,"%s: shape of the taper function\n",argv[0]);
      for(m=l=0;l<=lmax;l++)
	fprintf(stdout,"%g %g %i\n",
		(COMP_PRECISION)l/(COMP_PRECISION)lmax,
		amplitudefactor*
		taperf(l,lmax,lc,tapering,m,filter),l);
      break;
    }
    case VECABAB_POL_OUT:{
      if(!velin){
	fprintf(stderr,"%s: did not read pol/tor coefficients, output %i does not work\n",
		argv[0],VECABAB_POL_OUT);
	exit(-1);
      }else{
	fprintf(stderr,"%s: output of poloidal part of expansion\n",argv[0]);
	fprintf(stdout,"%i\n",lmax);
	for(l=0;l<=lmax;l++)
	  for(m=0;m<=l;m++){
	    os1 = POSLM(l, m);
	    fprintf(stdout,"%21.14e %21.14e\n",
		    *(amp+os1),*(bmp+os1));
	  }
      }
      break;
    }
    case VECABAB_TOR_OUT:{
      if(!velin){
	fprintf(stderr,"%s: did not read pol/tor coefficients, output %i does not work\n",
		argv[0],VECABAB_POL_OUT);
	exit(-1);
      }else{
	fprintf(stderr,"%s: output of toroidal part of expansion\n",argv[0]);
	fprintf(stdout,"%i\n",lmax);
	for(l=0;l<=lmax;l++)
	  for(m=0;m<=l;m++){
	    os1 = POSLM(l, m);
	    fprintf(stdout,"%21.14e %21.14e\n",
		    *(amt+os1),*(bmt+os1));
	  }
      }
      break;
    }
    case LMAB_OUT:{
      if(verbose)
	fprintf(stderr,"%s: physical convention l m A B format output, lmax=%i\n",argv[0],lmax);
      if(!velin){
	for(l=0;l<=lmax;l++)
	  for(m=0;m<=l;m++){
	    os1 = POSLM(l, m);
	    fprintf(stdout,"%i %i %21.14e %21.14e\n",l,m,
		    *(a+os1),*(b+os1));
	  }
      }else{
	fprintf(stdout,"%i\n",lmax);
	for(l=0;l<=lmax;l++)
	  for(m=0;m<=l;m++){
	    os1 = POSLM(l, m);
	    fprintf(stdout,"%i %i %21.14e %21.14e %21.14e %21.14e\n",
		    l,m,
		    *(amp+os1),*(bmp+os1),
		    *(amt+os1),*(bmt+os1));
	  }
      }
      break;
    } 
    case LMAB_GEODETIC_OUT:{
      if(verbose)
	fprintf(stderr,"%s: geodetic convention l m A B format output, lmax=%i\n",argv[0],lmax);
      if(!velin){
	for(l=0;l<=lmax;l++)
	  for(m=0;m<=l;m++){
	    os1 = POSLM(l, m);
	    fprintf(stdout,"%i %i %21.14e %21.14e\n",l,m,
		    *(a+os1)/GEODETIC_FACTOR(l,m), 
		    *(b+os1)/GEODETIC_FACTOR(l,m));
	  }
      }else{
	fprintf(stdout,"%i\n",lmax);
	for(l=0;l<=lmax;l++)
	  for(m=0;m<=l;m++){
	    os1 = POSLM(l, m);
	    fprintf(stdout,"%i %i %21.14e %21.14e %21.14e %21.14e\n",
		    l,m,
		    *(amp+os1)/GEODETIC_FACTOR(l,m),*(bmp+os1)/GEODETIC_FACTOR(l,m),
		    *(amt+os1)/GEODETIC_FACTOR(l,m),*(bmt+os1)/GEODETIC_FACTOR(l,m));
	  }
      }
      break;
    }
    case CORRL_OUT:{ /* calculate correlation 
			coefficient as a function of l 
			between two files */
      if(verbose)fprintf(stderr,"%s: correlation coefficients of each degree, lmax=%i\n",argv[0],lmax);
      for(l=1;l<=lmax;l++){
	tmp=correlation(a,b,c,d,l);
	if(finite(tmp))
	  fprintf(stdout,"%i %20.10lf\n",l,tmp);
	else
	  fprintf(stdout,"%i nan\n",l);
      }
      break;
    }
    case COORT_OUT:{ /* calculate total correlation coefficient between two files */
      if(verbose)fprintf(stderr,"%s: total correlation coefficient, summed over all l, lmax=%i\n",argv[0],lmax);
      fprintf(stdout,"%20.10lf\n",correlation(a,b,c,d,-lmax));
      break;
    }
    case VECABAB_OUT:{
      if(!velin){
	fprintf(stderr,"%s: velocity output requested, yet no velocity input read\n",argv[0]);
	exit(-1);
      }
      if(verbose)
	fprintf(stderr,"%s: poltor vector output in ABAB format (physical), lmax=%i\n",argv[0],lmax);
      fprintf(stdout,"%i\n",lmax);
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  fprintf(stdout,"%21.14e %21.14e %21.14e %21.14e\n",
		  *(amp+os1),*(bmp+os1),*(amt+os1),*(bmt+os1));
	}
      break;
    }
    case VECABAB_NEW_OUT:{
      if(!velin){
	fprintf(stderr,"%s: velocity output requested, yet no velocity input read\n",argv[0]);
	exit(-1);
      }
      if(verbose)
	fprintf(stderr,"%s: poltor vector output in new ABAB format (physical), lmax=%i\n",argv[0],lmax);
      fprintf(stdout,"%i %i %g %i %i %i\n",
	      lmax,0,0.0,1,2,0);
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  fprintf(stdout,"%21.14e %21.14e %21.14e %21.14e\n",
		  *(amp+os1),*(bmp+os1),*(amt+os1),*(bmt+os1));
	}
      break;
    }
    case VECAABBR_OUT:{
      if(!velin){
	fprintf(stderr,"%s: velocity output requested, yet no velocity input read\n",argv[0]);
	exit(-1);
      }
      if(verbose)fprintf(stderr,"%s: poltor vector output in ABAB format (Rick's convention), lmax=%i\n",argv[0],lmax);
      fprintf(stdout,"%i\n",lmax);
      for(l=0;l<=lmax;l++)
	for(m=0;m<=l;m++){
	  os1 = POSLM(l, m);
	  fprintf(stdout,"%21.14e %21.14e\n%21.14e %21.14e\n",
		  *(amp+os1)/RICK_FACTOR(l,m),
		  *(amt+os1)/RICK_FACTOR(l,m),
		  *(bmp+os1)/RICK_FACTOR(l,m),
		  *(bmt+os1)/RICK_FACTOR(l,m));
	}
      break;
    }
    default:{
      phelp(argv[0]);
      exit(-1);
      break;
    }}// end switch
  }// end scan of multiple input files loop
  return 0;
}
//
// apply taper here
//
COMP_PRECISION taperf(int l, int lmax, 
		      COMP_PRECISION limit, int mode,int m,
		      COMP_PRECISION *filter)
{
  /* should not taper in m since they form a full set */
  COMP_PRECISION lp,tmp,mp;
  lp=(COMP_PRECISION)l/(COMP_PRECISION)lmax;
  if((lp > 1.0)||(lp < 0)){
    fprintf(stderr,"taperf: errror l (%i) > lmax (%i) or smaller than zero\n",
	    l,lmax);
    exit(-1);
  }
  if(lp < limit){// are we to the left of the tapering limit?
    if((mode==L0_TAPER) && (l==0))// only l=0 taper does something then
      return 0.0;
    else
      return 1.0;
  }else{
    // rescale lp 
    if(limit==1.0){
      fprintf(stderr,"taperf: error: limit = 1.0\n");
      exit(-1);
    }
    lp = (lp-limit)/(1.0-limit);
    switch(mode){
    case PHI_ROTATE:
    case NNR_TAPER:
    case NO_TAPER:{
      return 1.0;
      break;
    }
    case COSSQR_TAPER:{
      tmp=cos(lp*PIHALF);
      return SQUARE(tmp);
      break;
    }
    case COSP4_TAPER:{
      tmp=cos(lp*PIHALF);
      tmp= SQUARE(tmp);
      return SQUARE(tmp);
      break;
    }
    case  ONEML_TAPER:{
      return 1.0-lp;
      break;
    }
    case ONEMLSQR_TAPER:{
      return 1.0-SQUARE(lp);
      break;
    }
    case SINOVRL_TAPER:{
      tmp=(lp!=0.0)?(sin(PI*lp)/(PI*lp)):(1.0);
      return tmp;
      break;
    }
    case SINOVRLM_TAPER:{/* that was probably used by Forte, does not make sense?! */
      tmp =((lp!=0.0)?(sin(PI*lp)/(PI*lp)):(1.0));
      mp=(COMP_PRECISION)m/(COMP_PRECISION)lmax;
      if(mp > limit){
	mp = (mp-limit)/(1.0-limit);// rescale mp
	tmp *= ((mp!=0.0)?(sin(PI*mp)/(PI*mp)):(1.0));
      }
      return tmp;
      break;
    }
    case ZERO_TAPER:{
      return 0.0;
      break;
    }
    case L0_TAPER:{
      if(l==0)
	return 0.0;
      else
	return 1.0;
      break;
    }
    case FROM_FILE_TAPER:{
      return(filter[l]);
      break;
    }
    default:{
      fprintf(stderr,"don't know taper number %i\n",
	      mode);
      exit(-1);
    }}
  }
  return(1/0);
}




void phelp(char *name)
{
  fprintf(stderr,"%s  [output fmt, %i] [tapering, %i] [ampl, %g] [input fmt, %i] [lmax, lmax] [lc, %g]\n",
	  name,OUT_FORMAT_DEFAULT,TAPERING_DEFAULT,AMPLITUDE_DEFAULT,IN_FORMAT_DEFAULT,LC_DEFAULT);
  fprintf(stderr,"\t  reads spherical harmonic coefficents A B  from stdin\n");
  fprintf(stderr,"\t  and converts the coefficients\n");
  fprintf(stderr,"\t  works for various combinations of normalizations and\n");
  fprintf(stderr,"\t  for scalar and vector fields\n");
  fprintf(stderr,"\t  Our internal convention is ""theoretical physics"", or ""physical norm"" as\n");
  fprintf(stderr,"\t  in Dahlen and Tromp (1998, B.8)\n");
  fprintf(stderr,"\t  output fmt:    Output format to stdout\n");
  fprintf(stderr,"\t                 %i: AB format, physical norm. (Dahlen & Tromp conv.)\n",
	  ABPHYS_OUT);
  fprintf(stderr,"\t                 %i: AB format, geodetic norm. (NASA conv.)\n",ABGEOD_OUT);
  fprintf(stderr,"\t                 %i: power per unit area and degree\n",POWER_OUT);
  fprintf(stderr,"\t                 %i: taper as a function of l (for debugging)\n",TAPER_OUT);
  fprintf(stderr,"\t                 %i: l m AB format, physical norm\n",LMAB_OUT);
  fprintf(stderr,"\t                 %i: correlation coefficient for degree 1<=l<=l_max, \n",CORRL_OUT);
  fprintf(stderr,"\t                    using two input files in AB format\n");
  fprintf(stderr,"\t                 %i: total correlation coefficient, \n",COORT_OUT);
  fprintf(stderr,"\t                    using two input files in AB format\n");
  fprintf(stderr,"\t                 %i: vector field, A_p B_p A_t B_t format (phys. norm)\n",VECABAB_OUT);
  fprintf(stderr,"\t                 %i: vector field, A_p A_t\\nB_p B_t format (Rick's norm)\n",
	  VECAABBR_OUT);
  fprintf(stderr,"\t                 %i: l m AB format (geodetic norm)\n",LMAB_GEODETIC_OUT);
  fprintf(stderr,"\t                 %i: RMS of expansion\n",TRMS_OUT);
  fprintf(stderr,"\t                 %i: total power or magnitude of expansion (include l=0 term)\n",
	  TPOWER_OUT);
  fprintf(stderr,"\t                 %i: vector field, physical norm, poloidal part only\n",
	  VECABAB_POL_OUT);
  fprintf(stderr,"\t                 %i: vector field, physical norm, toroidal part only\n",
	  VECABAB_TOR_OUT);
  fprintf(stderr,"\t                 %i: vector field, A_p B_p A_t B_t format (phys. norm), new format for HC\n",
	  VECABAB_NEW_OUT);
  fprintf(stderr,"\t  tapering:      %i: no tapering, l'=l/l_{max}\n",NO_TAPER);
  fprintf(stderr,"\t                 %i: cos(-pi/2*l')**2  tapering for l' >= lc \n",COSSQR_TAPER);
  fprintf(stderr,"\t                 %i: cos(-pi/2*l')**4  tapering for l' >= lc \n",COSP4_TAPER);
  fprintf(stderr,"\t                 %i: 1-l'              tapering for l' >= lc \n",ONEML_TAPER);
  fprintf(stderr,"\t                 %i: 1-(l')**2         tapering for l' >= lc \n",ONEMLSQR_TAPER);
  fprintf(stderr,"\t                 %i: sin(pi*l')/(pi*l')tapering for l' >= lc \n",SINOVRL_TAPER);
  fprintf(stderr,"\t                 %i: sin(pi*l')/(pi*l')tapering for l' >= lc, for m too (Forte)\n",
	  SINOVRLM_TAPER);
  fprintf(stderr,"\t                 %i: no net rotation for pol/tor AB\n",NNR_TAPER);
  fprintf(stderr,"\t                 %i: all coefficients l'>=lc set to zero\n",ZERO_TAPER);
  fprintf(stderr,"\t                 %i: l=0 term is set to zero (mean=0)\n",L0_TAPER);
  fprintf(stderr,"\t                 %i: read in w_0 ... w_{l_{max}} weights from file \"%s\"\n",
	  FROM_FILE_TAPER,FILTER_FILE);
  fprintf(stderr,"\t                     and scale all coefficients at l with the weight.\n");
  fprintf(stderr,"\t                     In this case, lc is set to zero by default.\n");

  fprintf(stderr,"\t                 %i: rotate field by amount dp, where dp is given as the amplification\n",
	  PHI_ROTATE);
  fprintf(stderr,"\t                     factor, in degrees. E.g. amp=10 means rotate by 10 degrees around\n");
  fprintf(stderr,"\t                     the rotation axis in easterly direction\n");

  fprintf(stderr,"\t  ampl:          amplification factor\n");

  fprintf(stderr,"\t  input fmt:     %i: AB format       physical norm.\n",AB_INPUT);
  fprintf(stderr,"\t                 %i: layer AB format geodetic norm.\n",LAB_GEOD_INPUT);
  fprintf(stderr,"\t                 %i: AB format       geodetic norm.\n",AB_GEOD_INPUT);
  fprintf(stderr,"\t                 %i: AB format       Rick ""fully norm""\n",AB_RICK_INPUT);
  fprintf(stderr,"\t                 %i: vector field, A_p B_p A_t B_t format (physical conv.)\n",
	  ABAB_INPUT);
  fprintf(stderr,"\t                 %i: vector field, A_p B_p A_t B_t format (geodetic conv.)\n",
	  ABAB_GEOD_INPUT);
  fprintf(stderr,"\t                 %i: vector field, A_p A_t \\n B_p B_t format (Rick's conv.)\n",
	  AABBR_INPUT);
  fprintf(stderr,"\t                 %i: l m A B format  geodetic (Rick)\n",LMAB_GEOD_INPUT);
  fprintf(stderr,"\t                 %i: interpolate two sets of coefficients\n",INTERPOLATE);
  fprintf(stderr,"\t                     using two input files in AB format\n");
  fprintf(stderr,"\t                     in this case, the ampl factor gives the weight of the\n");
  fprintf(stderr,"\t                     first file, the second is weighted with 1-ampl\n");
  fprintf(stderr,"\t                 %i: interpolate two sets of vector coefficients\n",INTERPOLATE_ABAB);
  fprintf(stderr,"\t                     using two input files in A_p B_p A_t B_t  format as in %i\n",
	  ABAB_INPUT);
  fprintf(stderr,"\t                     ampl factor meaning as for input mode %i\n",INTERPOLATE);
  fprintf(stderr,"\t                 %i: l m A B ""fully normalized"" (Edmonds, 1960) aka old Harvard\n",LMAB_FNORM_INPUT);
  fprintf(stderr,"\t                 %i: AB format  G. Master's ""fully norm"" (Edmonds, 1960\n",AB_MASTERS_INPUT);
  
  fprintf(stderr,"\t  lmax:          maximum degree of output, by default = input lmax\n");
  fprintf(stderr,"\t                 (if zero, use input lmax)\n");
  fprintf(stderr,"\t                 (if negative, set all coefficients SMALLER than lmax zero\n");

  fprintf(stderr,"\t  lc:            start of tapering (l/lmax has to be >=lc), lc is by default = %g,\n",LC_DEFAULT);
  fprintf(stderr,"\t                 except for filter type tapering, where it is set to zero.\n");            
  fprintf(stderr,"\n\n");
}




















