/* part of the shansyn spherical harmonics package, see COPYRIGHT for license */
/* $Id: powercorr.c,v 1.9 2001/09/19 20:21:13 becker Exp $ */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "function_macros.h"
#include "trig_constants.h"
#include "precision.h"
#include "spherical_harmonics_functions.h"
#include "legendre_macros.h"
/*

  calculates power per degree and unit area

 */

COMP_PRECISION degree_power(COMP_PRECISION *a,COMP_PRECISION *b, int l)
{
  COMP_PRECISION tmp;
  int m;
  // m=0
  tmp = SQUARE(*(a+POSLM(l, 0)));
  // 1<=m<=l
  for(m=1;m<=l;m++){
    tmp += SQUARE(*(a+POSLM(l, m)));
    tmp += SQUARE(*(b+POSLM(l, m)));
  }
  tmp /= (COMP_PRECISION)(2*l+1);
  return tmp;
}
/*
  calculate linear correlation coefficient between 
  two model expansions
*/
COMP_PRECISION correlation(COMP_PRECISION *a, COMP_PRECISION *b,
			   COMP_PRECISION *c, COMP_PRECISION *d, int l)
{
  COMP_PRECISION sum[4],tmp;
  int m,lmin,lmax;
  sum[0]=sum[1]=sum[2]=sum[3]=0.0;
  if(l<0){// sum over all l if l given negative
    lmin=  1;
    lmax= -l;
  }else{
    lmin = l;
    lmax = l;
  }
  for(l=lmin;l <= lmax;l++)
    for(m=0;m<=l;m++){
      sum[0]+= *(a+POSLM(l, m))* *(c+POSLM(l, m));
      sum[1]+= SQUARE(*(a+POSLM(l, m)));
      sum[2]+= SQUARE(*(c+POSLM(l, m)));
      if(m != 0){
	sum[0]+= *(b+POSLM(l, m))* *(d+POSLM(l, m));
	sum[1]+= SQUARE(*(b+POSLM(l, m)));
	sum[2]+= SQUARE(*(d+POSLM(l, m)));
      }
    }
  tmp=sqrt(sum[1]*sum[2]);
  return(sum[0]/tmp);
}


/*

  calculates correlation based on 1/(2l+1) coefficients
  (not good)

 */

COMP_PRECISION weighted_correlation(COMP_PRECISION *a, COMP_PRECISION *b,
				    COMP_PRECISION *c, COMP_PRECISION *d,int l)
{
  COMP_PRECISION sum[4],tmp,w,alm,blm,clm,dlm;
  int m,lmin,lmax;
  sum[0]=sum[1]=sum[2]=sum[3]=0.0;
  if(l<0){// sum over all l if l given negative
    lmin=  1;
    lmax= -l;
  }else{
    lmin = l;
    lmax = l;
  }
  for(l=lmin;l <= lmax;l++){
    w=1.0/(2.0*(COMP_PRECISION)l+1.0);
    for(m=0;m<=l;m++){
      alm=w* *(a+POSLM(l, m));
      clm=w* *(c+POSLM(l, m));
      sum[0]+= alm*clm;
      sum[1]+= SQUARE(alm);
      sum[2]+= SQUARE(clm);
      if(m!=0){
	blm=w* *(b+POSLM(l, m));
	dlm=w* *(d+POSLM(l, m));
	sum[0]+= blm*dlm;
	sum[1]+= SQUARE(blm);
	sum[2]+= SQUARE(dlm);
      }
    }
  }
  if((tmp=sqrt(sum[1]*sum[2])) > EPS_COMP_PREC)
    return(sum[0]/tmp);
  else
    return(0.0);
}


/*
  calculates RMS normalized by 
  surface area of sphere
*/
COMP_PRECISION calc_rms(COMP_PRECISION *a, COMP_PRECISION *b,int lmax)
{
  int l;
  COMP_PRECISION rms;
  rms=0.0;
  for(l=1;l<=lmax;l++)
    rms += ((COMP_PRECISION)(2*l+1))*degree_power(a,b,l);
  return(sqrt(rms)/TWO_SQRT_PI);
}
/*
  total power normalized by surface area of sphere
*/

COMP_PRECISION calc_total_power(COMP_PRECISION *a, COMP_PRECISION *b,int lmax)
{
  int l;
  COMP_PRECISION rms;
  rms=0.0;
  for(l=0;l<=lmax;l++)
    rms += ((COMP_PRECISION)(2*l+1))*degree_power(a,b,l);
  return(sqrt(rms)/TWO_SQRT_PI);
}

