/* part of the shansyn spherical harmonics package, see COPYRIGHT for license */
/* $Id: write_coefficients.c,v 1.7 2004/06/22 00:31:28 becker Exp $ */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "function_macros.h"
#include "trig_constants.h"
#include "precision.h"
#include "spherical_harmonics_functions.h"
#include "legendre_macros.h"


void write_coefficients(COMP_PRECISION *a,
			COMP_PRECISION *b,int lmax,
			COMP_PRECISION sfac,int print_lmax)
{
  int l,m,os;
  if(print_lmax)
    printf("%i\n",lmax);
  if(sfac != 1.0){
    for(l=0;l<=lmax;l++)
      for(m=0;m<=l;m++){
	os = POSLM(l,m);
	if((*(a+os) != 0)||(*(b+os)!=0))
	  printf(COEFF_DATA_FORMAT,*(a+os)*sfac,*(b+os)*sfac);
	else
	  printf("              0               0\n");
      }
  }else{
    for(l=0;l<=lmax;l++)
      for(m=0;m<=l;m++){
	os = POSLM(l,m);
	if((*(a+os) != 0)||(*(b+os)!=0))
	  printf(COEFF_DATA_FORMAT,*(a+os),*(b+os));
	else
	  printf("              0               0\n");
      }
  }
}
void write_model(struct mod *model,COMP_PRECISION fac)
{
  int i;
  switch(model->type){
  case DISCRETE:{
    printf("%i\n",model->n);// number of layers
    for(i=0;i<model->n;i++){
      printf("%g\n",model->d[i]);// depth
      // coefficients
      write_coefficients(model->a[i],model->b[i],
			 model->lmax,fac,TRUE);
    }
    break;
  }
  case CHEBYSHEV:{
    printf("%i\n",-model->n);// use negative order as layer number
    printf("%g %g\n",model->dmin,model->dmax);// boundaries
    for(i=0;i<model->n;i++){// coefficients
      write_coefficients(model->a[i],model->b[i],
			 model->lmax,fac,TRUE);
    }
    break;
  }
  case SPLINES:{
    printf("%i\n",model->n+1000);// use order + 1000 as layer number
    printf("%g %g\n",model->dmin,model->dmax);// boundaries
    for(i=0;i<model->n;i++){// coefficients
      write_coefficients(model->a[i],model->b[i],
			 model->lmax,fac,TRUE);
    }
    break;
  }
  case HC_TYPE:{
    for(i=0;i < model->n;i++){
      fprintf(stdout,"%i %i %g %i %i %i\n",
	      model->lmax,i,model->d[i],model->n,1,0);
      write_coefficients(model->a[i],model->b[i],
			 model->lmax,fac,FALSE);
    }
    break;
  }
  default:{
    fprintf(stderr,"write_model: type %i undefined\n",
	    model->type);
    exit(-1);
  }}
}
